#!/usr/bin/env python

import os
from plasTeX.ConfigManager import *

c = config = ConfigManager()

#
# General
#
general = c.add_section('general')

general['renderer'] = StringOption(
    """ Renderer to use for conversion """,
    options = '--renderer',
    default = 'XHTML',
)

general['theme'] = StringOption(
    """ Theme for the renderer to use """,
    options = '--theme',
    default = 'default',
)

general['copy-theme-extras'] = BooleanOption(
    """  Copy files associated with the theme to the output directory """,
    options = '--copy-theme-extras !--no-theme-extras',
    default = True,
)

general['kpsewhich'] = StringOption(
    """ Program which locates LaTeX files and packages """,
    options = '--kpsewhich',
    default = 'kpsewhich',
)

general['xml'] = BooleanOption(
    """ Dump XML representation of the document (for debugging) """,
    options = '--xml',
    default = False,
)

def readconfig(file):
    """ Read a configuration file """
    if not os.path.isfile(file):
        print("WARNING: Could not load config file '%s'" % file)
        return
    config.read(file)

general['config'] = StringOption(
    """
    Load additional configuration file

    This configuration file will be loaded during the processing of
    command-line options.  However, configuration files cannot override
    command-line options.  So no matter what is in the configuration file,
    if an option is specified on the command-line, it will always win.
    To eliminate any confusion, this should generally be the first
    argument.

    """,
    options = '--config -c',
    callback = readconfig,
)

general['paux-dirs'] = MultiOption(
    """
    Directories where *.paux files should be loaded from.

    """,
    options = '--paux-dirs',
)

#
# Links
#

links = c.add_section('links')

def getstring(data):
    """ Return a string which may or may not be quoted """
    data = list(data.lstrip())

    if not data:
        return ''

    s = []
    for quote in '\'"':
        if data[0] == quote:
            data.pop(0)
            while data:
                if data[0] == quote:
                    data.pop(0)
                    break
                s.append(data.pop(0))
            return ''.join(s), ''.join(data)

    while data:
        if data[0] != ' ':
            s.append(data.pop(0))
            continue
        break
    return ''.join(s), ''.join(data)


def setlinks(data):
    """ Set links in the configuration """
    data = data[1:-1].strip()
    key, data = getstring(data)
    url, data = getstring(data)
    title, data = getstring(data)
    if not title:
        title = url
        url = ''
    if title:
        links['%s-title' % key] = StringOption(default=title)
    if url:
        links['%s-url' % key] = StringOption(default=url)

links[';links'] = CompoundOption(
    """ Set links for use in navigation """,
    options = '--link',
    category = 'document',
    callback = setlinks,
)

#
# Counters
#

counters = c.add_section('counters')

def setcounter(data):
    """ Set counters in the configuration """
    value = [x for x in data[1:-1].split() if x]
    if len(value) == 2:
        counters[value[0]] = IntegerOption(default=int(value[1]))

counters[';counters'] = CompoundOption(
    """ Set initial counter values """,
    options = '--counter',
    category = 'document',
    callback = setcounter,
)

#
# Files
#

files = c.add_section('files')
c.add_category('files', 'File Handling Options')

files['input-encoding'] = StringOption(
    """ Input file encoding """,
    options = '--input-encoding',
    default = 'utf-8',
    category = 'files',
)

files['output-encoding'] = StringOption(
    """ Output file encoding """,
    options = '--output-encoding',
    default = 'utf-8',
    category = 'files',
)

files['escape-high-chars'] = BooleanOption(
    """ Escape characters that are higher than 7-bit """,
    options = '--escape-high-chars',
    default = False,
    category = 'files',
)

files['split-level'] = IntegerOption(
    """ Highest section level that generates a new file """,
    options = '--split-level',
    default = 2,
    category = 'files',
)

files['log'] = BooleanOption(
    """ Log messages go to log file instead of console """,
    options = '--log',
    default = False,
    category = 'files',
)

def setFilename(data):
    """ If there is only one filename specified, turn off splitting """
    data = data.strip()
    if ' ' in data:
        return data
    if '[' in data:
        return data
    files['split-level'] = -10
    return data

files['filename'] = StringOption(
    """ Template for output filenames """,
    options = '--filename',
    default = 'index [$id, sect$num(4)]',
    category = 'files',
    callback = setFilename,
)

files['bad-chars'] = StringOption(
    """ Characters that should not be allowed in a filename """,
    options = '--bad-filename-chars',
    default = ': #$%^&*!~`"\'=?/{}[]()|<>;\\,.',
    category = 'files',
)

files['bad-chars-sub'] = StringOption(
    """ Character that should be used instead of an illegal character """,
    options = '--bad-filename-chars-sub',
    default = '-',
    category = 'files',
)

files['directory'] = StringOption(
    """ Directory to put output files into """,
    options = '--dir -d',
    category = 'files',
    default = '$jobname',
)

#
# Images
#

images = c.add_section('images')
c.add_category('images', 'Image Generation Options')

images['base-url'] = StringOption(
    """ Base URL for all images """,
    options = '--image-base-url',
    category = 'images',
)

images['enabled'] = BooleanOption(
    """ Enable image generation """,
    options = '--enable-images !--disable-images',
    default = True,
    category = 'images',
)

images['imager'] = StringOption(
    """ LaTeX to image program """,
    options = '--imager',
    default = 'dvipng dvi2bitmap pdftoppm gspdfpng gsdvipng OSXCoreGraphics',
    category = 'images',
)

images['vector-imager'] = StringOption(
    """ LaTeX to vector image program """,
    options = '--vector-imager',
    default = 'none dvisvgm',
    category = 'images',
)

images['filenames'] = StringOption(
    """ Template for image filenames """,
    options = '--image-filenames',
    default = 'images/img-$num(4)',
    category = 'images',
)

images['baseline-padding'] = IntegerOption(
    """ Amount to pad the image below the baseline """,
    options = '--image-baseline-padding',
    default = 0,
    category = 'images',
)

images['scale-factor'] = FloatOption(
    """ Factor to scale externally included images by """,
    options = '--image-scale-factor',
    default = 1.0,
    category = 'images',
)

images['compiler'] = StringOption(
    """ LaTeX command to use when compiling image document """,
    options = '--image-compiler',
    category = 'images',
)

images['cache'] = BooleanOption(
    """  Enable image caching between runs """,
    options = '--enable-image-cache !--disable-image-cache',
    default = False,
    category = 'images',
)

images['save-file'] = BooleanOption(
    """ Should the temporary images.tex file be saved for debugging? """,
    options = '--save-image-file !--delete-image-file',
    default = False,
    category = 'images',
)

images['transparent'] = BooleanOption(
    """ Specifies whether the image backgrounds should be transparent or not """,
    options = '--transparent-images !--opaque-images',
    default = False,
    category = 'images',
)

images['resolution'] = IntegerOption(
    """ Resolution of images document """,
    options = '--image-resolution',
    default = 0,
    category = 'images',
)

#
# Document
#

doc = c.add_section('document')
c.add_category('document', 'Document Options')

doc['base-url'] = StringOption(
    """ Base URL for inter-node links """,
    options = '--base-url',
    category = 'document',
)

doc['title'] = StringOption(
    """
    Title for the document

    This option specifies a title to use instead of the title
    specified in the LaTeX document.

    """,
    options = '--title',
    category = 'document',
)

doc['toc-depth'] = IntegerOption(
    """ Number of levels to display in the table of contents """,
    options = '--toc-depth',
    category = 'document',
    default = 3,
)

doc['toc-non-files'] = BooleanOption(
    """ Display sections that do not create files in the table of contents """,
    options = '--toc-non-files',
    category = 'document',
    default = False,
)

doc['sec-num-depth'] = IntegerOption(
    """ Maximum section depth to display section numbers """,
    options = '--sec-num-depth',
    category = 'document',
    default = 2,
)

doc['index-columns'] = IntegerOption(
    """ Number of columns to split the index entries into """,
    options = '--index-columns',
    category = 'document',
    default = 2,
)

doc['lang-terms'] = StringOption(
    """ Specifies a '%s' delimited list of files that contain language terms """ % os.pathsep,
    options = '--lang-terms',
    category = 'document',
    default = '',
)

config.read('~/.plasTeXrc')
config.read('/usr/local/etc/plasTeXrc')
config.read(os.path.join(os.path.dirname(__file__), 'plasTeXrc'))

del c



#!/usr/bin/env python

import os
import re
import subprocess

from plasTeX.Imagers import Imager as _Imager

class pdftoppm(_Imager):
    """ Imager that uses libpoppler's pdftoppm to convert pdf to png """
    command = 'pdftoppm -png -r 150'
    compiler = 'pdflatex'
    fileExtension = '.png'

    def verify(self):
        """
        pdftoppm writes its help message to standard error, and not
        standard output as plasTeX expects, and we thus have to
        override this method. Note that this method will also work if
        pdftoppm writes its help message to standard output.

        """
        cmd = self.command.split()[0]
        proc = subprocess.Popen('%s --help' % cmd,
                                shell=True,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT,
                                universal_newlines=True)
        return 'pdftoppm' in proc.communicate()

    def executeConverter(self, output):
        """
        We need to override this because plasTeX always puts the input
        file at the end of the command-line.  We also need to return the
        list of images.

        """
        open('images.out', 'wb').write(output.read())
        options = ''
        if self._configOptions:
            for opt, value in self._configOptions:
                opt, value = str(opt), str(value)
                if ' ' in value:
                    value = '"%s"' % value
                options += '%s %s ' % (opt, value)
        rc = os.system('%s %s%s img' % (self.command, options, 'images.out'))
        return rc, [f for f in os.listdir('.') if re.match(r'^img-\d+\.\w+$', f)]

Imager = pdftoppm



#!/usr/bin/env python

#from Entities import *

from plasTeX.Base.LaTeX.Accents import *
from plasTeX.Base.LaTeX.Alignment import *
from plasTeX.Base.LaTeX.Arrays import *
from plasTeX.Base.LaTeX.Bibliography import *
from plasTeX.Base.LaTeX.Boxes import *
from plasTeX.Base.LaTeX.Breaking import *
from plasTeX.Base.LaTeX.Characters import *
from plasTeX.Base.LaTeX.Crossref import *
from plasTeX.Base.LaTeX.Definitions import *
from plasTeX.Base.LaTeX.Document import *
from plasTeX.Base.LaTeX.Environments import *
from plasTeX.Base.LaTeX.FontSelection import *
from plasTeX.Base.LaTeX.Footnotes import *
from plasTeX.Base.LaTeX.Files import *
from plasTeX.Base.LaTeX.Floats import *
from plasTeX.Base.LaTeX.Index import *
from plasTeX.Base.LaTeX.Lengths import *
from plasTeX.Base.LaTeX.Lists import *
from plasTeX.Base.LaTeX.Math import *
from plasTeX.Base.LaTeX.Numbering import *
from plasTeX.Base.LaTeX.Packages import *
from plasTeX.Base.LaTeX.Pictures import *
from plasTeX.Base.LaTeX.Paragraphs import *
from plasTeX.Base.LaTeX.Quotations import *
from plasTeX.Base.LaTeX.Sectioning import *
from plasTeX.Base.LaTeX.Sentences import *
from plasTeX.Base.LaTeX.Space import *
from plasTeX.Base.LaTeX.Tabbing import *
from plasTeX.Base.LaTeX.Verbatim import *

from plasTeX import Command

class ifundefined_(Command):
    macroName = '@ifundefined'
    args = 'name:str true:nox false:nox'
    def invoke(self, tex):
        a = self.parse(tex)
        if a['name'] in list(self.ownerDocument.context.keys()):
            tex.pushTokens(a['false'])
        else:
            tex.pushTokens(a['true'])
        return []

class vwritefile_(Command):
    macroName = '@vwritefile'
    args = 'file:nox content:nox'

class pagelabel(Command):
    args = 'label:nox content:nox'

class verbatiminput(Command):
    pass

class makeatother(Command):
    def invoke(self, tex):
        self.ownerDocument.context.catcode('@', 12)

class makeatletter(Command):
    def invoke(self, tex):
        self.ownerDocument.context.catcode('@', 11)



#!/usr/bin/env python

"""
C.12 Line and Page Breaking (p212)

"""

from plasTeX import Command, Environment
from plasTeX.Logging import getLogger

#
# C.12.1
#

class linebreak(Command):
    args = '[ num:int ]'

class nolinebreak(Command):
    args = '[ num:int ]'


class newline(Command):
    pass

class NewLine(newline):
    macroName = '\\'
    args = '* [ len:dimen ]'

class AllowHyphen(Command):
    macroName = '-'

class hyphenation(Command):
    args = 'words:str'

class sloppy(Command):
    pass

class fussy(Command):
    pass

class sloppypar(Command):
    pass

#
# C.12.2 Page Breaking
#

class pagebreak(Command):
    args = '[ num:int ]'

class nopagebreak(Command):
    args = '[ num:int ]'

class enlargethispage(Command):
    args = '* len:dimen'

class newpage(Command):
    pass

class clearpage(Command):
    pass

class cleardoublepage(Command):
    pass




#!/usr/bin/env python

import string
from plasTeX.DOM import Node, Text
from plasTeX import encoding
from io import BytesIO, StringIO, TextIOWrapper

# Default TeX categories
DEFAULT_CATEGORIES = [
   '\\',  # 0  - Escape character
   '{',   # 1  - Beginning of group
   '}',   # 2  - End of group
   '$',   # 3  - Math shift
   '&',   # 4  - Alignment tab
   '\n',  # 5  - End of line
   '#',   # 6  - Parameter
   '^',   # 7  - Superscript
   '_',   # 8  - Subscript
   '\x00',# 9  - Ignored character
   ' \t\r\f', # 10 - Space
   encoding.stringletters() + '@', # - Letter
   '',    # 12 - Other character - This isn't explicitly defined.  If it
          #                        isn't any of the other categories, then
          #                        it's an "other" character.
   '~',   # 13 - Active character
   '%',   # 14 - Comment character
   ''     # 15 - Invalid character
]

VERBATIM_CATEGORIES = [''] * 16
VERBATIM_CATEGORIES[11] = encoding.stringletters()

class EndInput(Exception):
    pass

class Token(Text):
    """ Base class for all TeX tokens """

    # The 16 category codes defined by TeX
    CC_ESCAPE = 0
    CC_BGROUP = 1
    CC_EGROUP = 2
    CC_MATHSHIFT = 3
    CC_ALIGNMENT = 4
    CC_EOL = 5
    CC_PARAMETER = 6
    CC_SUPER = 7
    CC_SUB = 8
    CC_IGNORED = 9
    CC_SPACE = 10
    CC_LETTER = 11
    CC_OTHER = 12
    CC_ACTIVE = 13
    CC_COMMENT = 14
    CC_INVALID = 15

    TOKEN_SLOTS = __slots__ = Text.TEXT_SLOTS

    catcode = None       # TeX category code
    macroName = None     # Macro to invoke in place of this token

    def __repr__(self):
        return self.source

    def __eq__(self, other):
        if isinstance(other, Token):
            return (self is other
            or (self.catcode == other.catcode and super(Token,self).__eq__(other)))
        return super(Token,self).__eq__(str(other))

    def __ne__(self, other):
        try:
            return self.catcode != other.catcode or super(Token,self).__ne__(other)
        except AttributeError:
            return NotImplemented

    def __hash__(self):
        return super(Token,self).__hash__()

    def __lt__(self, other):
        if isinstance(other, Token):
            if self.catcode == other.catcode:
                return str(self) < str(other)
            else:
                return self.catcode <  other.catcode
        else:
            return str(self) <  str(other)

    def __str__(self):
        return self.encode('utf-8').decode('utf-8')


    @property
    def source(self):
        return self


class EscapeSequence(Token):
    """
    Escape sequence token

    This token represents a TeX escape sequence.  Doing str(...)
    on this token returns the name of the escape sequence without
    the escape character.

    """
    catcode = Token.CC_ESCAPE
    @property
    def source(self):
        if self == 'par':
            return '\n\n'
        # Handle active character format
        if '::' in self:
            return self.split('::').pop()
        return '\\%s ' % self
    @property
    def macroName(self):
        return self
    __slots__ = Token.TOKEN_SLOTS

class BeginGroup(Token):
    """ Beginning of a TeX group """
    catcode = Token.CC_BGROUP
    macroName = 'bgroup'
    __slots__ = Token.TOKEN_SLOTS

class EndGroup(Token):
    """ Ending of a TeX group """
    catcode = Token.CC_EGROUP
    macroName = 'egroup'
    __slots__ = Token.TOKEN_SLOTS

class MathShift(Token):
    catcode = Token.CC_MATHSHIFT
    macroName = 'active::$'
    __slots__ = Token.TOKEN_SLOTS

class Alignment(Token):
    catcode = Token.CC_ALIGNMENT
    macroName = 'active::&'
    __slots__ = Token.TOKEN_SLOTS

class EndOfLine(Token):
    catcode = Token.CC_EOL
    isElementContentWhitespace = True
    __slots__ = Token.TOKEN_SLOTS

class Parameter(Token):
    catcode = Token.CC_PARAMETER
    __slots__ = Token.TOKEN_SLOTS

class Superscript(Token):
    catcode = Token.CC_SUPER
    macroName = 'active::^'
    __slots__ = Token.TOKEN_SLOTS

class Subscript(Token):
    catcode = Token.CC_SUB
    macroName = 'active::_'
    __slots__ = Token.TOKEN_SLOTS

class Space(Token):
    catcode = Token.CC_SPACE
    isElementContentWhitespace = True
    __slots__ = Token.TOKEN_SLOTS

class Letter(Token):
    catcode = Token.CC_LETTER
    __slots__ = Token.TOKEN_SLOTS

class Other(Token):
    catcode = Token.CC_OTHER
    __slots__ = Token.TOKEN_SLOTS

class Active(Token):
    catcode = Token.CC_ACTIVE
    __slots__ = Token.TOKEN_SLOTS

class Comment(Token):
    catcode = Token.CC_COMMENT
    nodeType = Node.COMMENT_NODE
    nodeName = '#comment'
    isElementContentWhitespace = True
    __slots__ = Token.TOKEN_SLOTS

class Tokenizer(object):

    # Tokenizer states
    STATE_S = 1
    STATE_M = 2
    STATE_N = 4

    # Array for getting token class for the corresponding catcode
    tokenClasses = [None] * 16
    tokenClasses[Token.CC_ESCAPE] = EscapeSequence
    tokenClasses[Token.CC_BGROUP] = BeginGroup
    tokenClasses[Token.CC_EGROUP] = EndGroup
    tokenClasses[Token.CC_MATHSHIFT] = MathShift
    tokenClasses[Token.CC_ALIGNMENT] = Alignment
    tokenClasses[Token.CC_EOL] = EndOfLine
    tokenClasses[Token.CC_PARAMETER] = Parameter
    tokenClasses[Token.CC_SUPER] = Superscript
    tokenClasses[Token.CC_SUB] = Subscript
    tokenClasses[Token.CC_SPACE] = Space
    tokenClasses[Token.CC_LETTER] = Letter
    tokenClasses[Token.CC_OTHER] = Other
    tokenClasses[Token.CC_ACTIVE] = Active
    tokenClasses[Token.CC_COMMENT] = Comment

    def __init__(self, source, context):
        """
        Instantiate a tokenizer

        Required Arguments:
        source -- the source to tokenize.  This can be a string containing
            TeX source, a file object that contains TeX source, or a
            list of tokens
        context -- the document's context object

        """
        self.context = context
        self.state = Tokenizer.STATE_N
        self._charBuffer = []
        self._tokBuffer = []
        if isinstance(source, str):
            source = StringIO(source)
            self.filename = '<string>'
        elif isinstance(source, bytes):
            source = TextIOWrapper(BytesIO(source), encoding='utf-8')
            self.filename = '<string>'
        elif isinstance(source, (tuple,list)):
            self.pushTokens(source)
            source = StringIO('')
            self.filename = '<tokens>'
        else:
            self.filename = source.name
        self.seek = source.seek
        self.read = source.read
        self.readline = source.readline
        self.tell = source.tell
        self.lineNumber = 1

# There seems to be a problem with readline in Python 2.4 !!!
    def readline(self):
        read = self.read
        mybuffer = self._charBuffer
        while 1:
            if mybuffer:
                char = mybuffer.pop(0)
            else:
                char = read(1)
            if not char or ord(char) == 10:
                break

    def iterchars(self):
        """
        Get the next character in the stream and its category code

        This function handles automatically converts characters like
        ^^M, ^^@, etc. into the correct character.  It also bypasses
        ignored and invalid characters.

        If you are iterating through the characters in a TeX instance
        and you go too far, you can put the character back with
        the pushChar() method.

        """
        # Create locals before going into the generator loop
        mybuffer = self._charBuffer
        classes = self.tokenClasses
        read = self.read

        whichCode = self.context.whichCode
        CC_SUPER = Token.CC_SUPER
        CC_IGNORED = Token.CC_IGNORED
        CC_INVALID = Token.CC_INVALID

        def _read1():
            if mybuffer:
                return mybuffer.pop(0)
            return read(1)

        while True:
            token = _read1()

            if not token:
                break

            if token == '\n':
                self.lineNumber += 1
                self.context.meta["lines"] += 1

            code = whichCode(token)

            if code == CC_SUPER:

                # Handle characters like ^^M, ^^@, etc.
                next_char = _read1()

                if next_char != token:
                    self.pushChar(next_char)
                else:
                    next_char = _read1()
                    num = ord(next_char)
                    if num >= 64:
                        token = chr(num-64)
                    else:
                        token = chr(num+64)
                    code = whichCode(token)

            # Just go to the next character if you see one of these...
            if code == CC_IGNORED or code == CC_INVALID:
                continue

            yield classes[code](token)

    def pushChar(self, char):
        """
        Push a character back into the stream to be re-read

        Required Arguments:
        char -- the character to push back

        """
        self._charBuffer.insert(0, char)

    def pushToken(self, token):
        """
        Push a token back into the stream to be re-read

        Required Arguments:
        token -- token to be pushed back

        """
        if token is not None:
            self._tokBuffer.insert(0, token)

    def pushTokens(self, tokens):
        """
        Push a list of tokens back into the stream to be re-read

        Required Arguments:
        tokens -- list of tokens to push back

        """
        if tokens:
            tokens = list(tokens)
            tokens.reverse()
            for t in tokens:
                self.pushToken(t)

    def __iter__(self):
        """
        Iterate over tokens in the input stream

        Returns:
        generator that iterates through tokens in the stream

        """
        # Cache variables to prevent globol lookups during generator
        global Space, EscapeSequence
        Space = Space
        EscapeSequence = EscapeSequence
        mybuffer = self._tokBuffer
        charIter = self.iterchars()
        context = self.context
        pushChar = self.pushChar
        STATE_N = self.STATE_N
        STATE_M = self.STATE_M
        STATE_S = self.STATE_S
        ELEMENT_NODE = Node.ELEMENT_NODE
        CC_LETTER = Token.CC_LETTER
        CC_OTHER = Token.CC_OTHER
        CC_SPACE = Token.CC_SPACE
        CC_EOL = Token.CC_EOL
        CC_ESCAPE = Token.CC_ESCAPE
        CC_EOL = Token.CC_EOL
        CC_COMMENT = Token.CC_COMMENT
        CC_ACTIVE = Token.CC_ACTIVE
        prev = None

        while 1:

            # Purge mybuffer first
            while mybuffer:
                yield mybuffer.pop(0)

            # Get the next character
            try:
                token = next(charIter)
            except StopIteration:
                raise EndInput

            if token.nodeType == ELEMENT_NODE:
                raise ValueError('Expanded tokens should never make it here')

            code = token.catcode

            # Short circuit letters and other since they are so common
            if code == CC_LETTER or code == CC_OTHER:
                self.state = STATE_M

            # Whitespace
            elif code == CC_SPACE:
                if self.state  == STATE_S or self.state == STATE_N:
                    continue
                self.state = STATE_S
                token = Space(' ')

            # End of line
            elif code == CC_EOL:
                state = self.state
                if state == STATE_S:
                    self.state = STATE_N
                    continue
                elif state == STATE_M:
                    token = Space(' ')
                    code = CC_SPACE
                    self.state = STATE_N
                elif state == STATE_N:
                    # ord(token) != 10 is the same as saying token != '\n'
                    # but it is much faster.
                    if ord(token) != 10:
                        self.lineNumber += 1
                        self.readline()
                    token = EscapeSequence('par')
                    # Prevent adjacent paragraphs
                    if prev == token:
                        continue
                    code = CC_ESCAPE

            # Escape sequence
            elif code == CC_ESCAPE:

                # Get name of command sequence
                self.state = STATE_M

                for token in charIter:

                    if token.catcode == CC_LETTER:
                        word = [token]
                        for t in charIter:
                            if t.catcode == CC_LETTER:
                                word.append(t)
                            else:
                                pushChar(t)
                                break
                        token = EscapeSequence(''.join(word))

                    elif token.catcode == CC_EOL:
                        #pushChar(token)
                        #token = EscapeSequence()
                        token = Space(' ')
                        self.state = STATE_S

                    else:
                        token = EscapeSequence(token)
#
# Because we can implement macros both in LaTeX and Python, we don't
# always want the whitespace to be eaten.  For example, implementing
# \chardef\%=`% would be \char{`%} in TeX, but in Python it's just
# another macro class that would eat whitspace incorrectly.  So we
# have to do this kind of thing in the parse() method of Macro.
#
                    if token.catcode != CC_EOL:
# HACK: I couldn't get the parse() thing to work so I'm just not
#       going to parse whitespace after EscapeSequences that end in
#       non-letter characters as a half-assed solution.
                        if token[-1] in encoding.stringletters():
                            # Absorb following whitespace
                            self.state = STATE_S

                    break

                else: token = EscapeSequence()

                # Check for any \let aliases
                token = context.lets.get(token, token)

                # TODO: This action should be generalized so that the
                #       tokens are processed recursively
                if token is not token and token.catcode == CC_COMMENT:
                    self.readline()
                    self.lineNumber += 1
                    self.state = STATE_N
                    continue

            elif code == CC_COMMENT:
                self.readline()
                self.lineNumber += 1
                self.state = STATE_N
                continue

            elif code == CC_ACTIVE:
                token = EscapeSequence('active::%s' % token)
                token = context.lets.get(token, token)
                self.state = STATE_M

            else:
                self.state = STATE_M

            prev = token

            yield token



import pdb

from plasTeX import Command
from plasTeX.Logging import getLogger


class settrace(Command):
    def invoke(self, tex):
        document = self.ownerDocument
        config = document.config
        context = document.context
        pdb.set_trace()


class setloglevel(Command):
    args = 'logger:str level:str'

    def invoke(self, tex):
        self.parse(tex)
        logger_name = self.attributes['logger']
        level = self.attributes['level']
        logger = getLogger(logger_name)
        logger.setLevel(level)

        config = self.ownerDocument.config
        config['logging'].data[logger_name].setValue(level)


def ProcessOptions(options, document):
    if 'post_parse_trace' in options:
        def trace_callback():
            config = document.config
            context = document.context
            pdb.set_trace()
        document.postParseCallbacks.append(trace_callback)



#!/usr/bin/env python

"""
This package is not properly implemented.  The \ifthenelse command simply
returns the 'false' portion of the command.  Hopefully, some day this will
be done properly.

"""

from plasTeX import Command

class ifthenelse(Command):
    args = 'test:nox then:nox else' 

    class _not(Command):
        macroName = 'not'

    class _and(Command):
        macroName = 'and'

    class _or(Command):
        macroName = 'or'

    class NOT(Command):
        pass

    class AND(Command):
        pass

    class OR(Command):
        pass

    class openParen(Command):
        macroName = '('

    class closeParen(Command):
        macroName = ')'

    class isodd(Command):
        args = 'number:int'

    class isundefined(Command):
        args = 'command:str'

    class equal(Command):
        args = 'first second'

    class lengthtest(Command):
        args = 'test'

    class boolean(Command):
        args = 'name:str'

class newboolean(Command):
    args = 'name:str'

class provideboolean(Command):
    args = 'name:str'

class setboolean(Command):
    args = 'name:str value:str'

class whiledo(Command):
    args = 'test:nox operations'






#!/usr/bin/env python

"""
C.8 Definitions, Numbering, and Programming

"""

from plasTeX import Command, Environment
from plasTeX.Logging import getLogger

log = getLogger()
status = getLogger('status')
deflog = getLogger('parse.definitions')
envlog = getLogger('parse.environments')

#
# C.8.1 Defining Commands
#

class newcommand(Command):
    """ \\newcommand """
    args = '* name:cs [ nargs:int ] [ opt:nox ] definition:nox'
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        args = (a['name'], a['nargs'], a['definition'])
        kwargs = {'opt':a['opt']}
        deflog.debug('command %s %s %s', *args)
        self.ownerDocument.context.newcommand(*args, **kwargs)

class renewcommand(newcommand):
    pass

class providecommand(newcommand):
    pass

class DeclareRobustCommand(newcommand):
    pass

class DeclareTextCommandDefault(newcommand):
    pass

#
# C.8.2 Defining Environments
#

class newenvironment(Command):
    """ \\newenvironment """
    args = '* name:str [ nargs:int ] [ opt:nox ] begin:nox end:nox'
    def invoke(self, tex):
        self.parse(tex)
        a = self.attributes
        args = (a['name'], a['nargs'], [a['begin'], a['end']])
        kwargs = {'opt':a['opt']}
        deflog.debug('environment %s %s %s', *args)
        self.ownerDocument.context.newenvironment(*args, **kwargs)

class renewenvironment(newenvironment):
    pass


#
# C.8.3 Theorem-like Environments
#

class newtheorem(Command):
    args = '* name:str [ counter:str ] caption [ within:str ]'
    def invoke(self, tex):
        self.parse(tex)
        attrs = self.attributes
        name = attrs['name']
        counter = attrs['counter']
        caption = attrs['caption']
        within = attrs['within']
        if not counter and not attrs['*modifier*']:
            counter = name
            if within:
                self.ownerDocument.context.newcounter(counter,initial=0,resetby=within, format='${the%s}.${%s}' % (within, name))
            else:
                self.ownerDocument.context.newcounter(counter,initial=0)
        deflog.debug('newtheorem %s', name)

        # The nodeName key below ensure all theorem type will call the same
        # rendering method, the type of theorem being retained in the thmName
        # attribute
        if attrs['*modifier*']:
            newclass = type(str(name), (Environment,),
                    {'caption': caption, 'nodeName': 'thmenv', 'thmName': name,
                        'args': '[title]', 'forcePars': True})
        else:
            newclass = type(str(name), (Environment,),
                    {'caption': caption, 'nodeName': 'thmenv', 'thmName': name,
                        'counter': counter, 'args': '[title]', 'forcePars': True})
        self.ownerDocument.context.addGlobal(name, newclass)


class proof(Environment):
    blockType = True
    args ='[caption]'

    def digest(self, tokens):
        Environment.digest(self, tokens)
        self.caption = self.attributes.get('caption', '')



import os, inspect, re, shutil

from plasTeX.Logging import getLogger

log = getLogger()

def rendererDir(renderer):
    """Convenience wrapper around inspect.getfile to get the renderer
    directory"""
    return os.path.dirname(inspect.getfile(renderer.__class__))

class PackageResource(object):
    outdir = ''
    copy = False
    key=''

    def __init__(self, renderers=None, package='', key='', data=None):
        if isinstance(renderers, list):
            self.rendererPatterns = renderers
        else:
            self.rendererPatterns = [renderers]
        self.package = package
        self.data = data
        self.key = key or self.key
        self.workingdir = ''

    def alter(self, renderer=None, rendererName='', document=None, target=''):
        doIt = False
        for pattern in self.rendererPatterns:
            if re.match(pattern, rendererName):
                doIt = True
                break
        if doIt:
            self.workingdir = document.userdata.get('working-dir', '')
            self.alterRenderer(renderer)
            self.alterDocument(document=document, rendererName=rendererName)
            self.copyFile(renderer=renderer, target=target)

    def alterRenderer(self, renderer):
        pass

    def alterDocument(self, document=None, rendererName=''):
        rendererdata = document.rendererdata[rendererName]
        if self.key:
            if isinstance(self.data, list):
                data = self.data
            else:
                data = [self.data]
            rendererdata.setdefault(self.key, []).extend(data)

    def copyFile(self, renderer=None, target=None):
        if self.copy and renderer and target:
            rendererdir = rendererDir(renderer)
            source = os.path.join(self.workingdir, self.package, self.data)
            try:
                shutil.copy(
                        source,
                        os.path.join(target, self.outdir))
            except IOError:
                source = os.path.join(rendererdir, self.package, self.data)
                try:
                    shutil.copy(
                            source,
                            os.path.join(target, self.outdir))
                except IOError:
                    log.error('Package resource file not found :'+source)


class PackageCss(PackageResource):
    outdir = 'styles'
    key = 'css'
    copy = True

class PackageJs(PackageResource):
    outdir = 'js'
    key = 'js'
    copy = True

class PackageTemplateDir(PackageResource):
    def alterRenderer(self, renderer):
        rendererdir = rendererDir(renderer)
        subdir = os.path.join(self.package, self.data or '')
        userpath = os.path.join(self.workingdir, subdir)
        rendererpath = os.path.join(rendererdir, subdir)
        if os.path.isdir(userpath):
            renderer.importDirectory(userpath)
        elif os.path.isdir(rendererpath):
            renderer.importDirectory(rendererpath)
        else:
            log.error('Package template directory not found :' + subdir)



#!/usr/bin/env python

import re
from plasTeX.ConfigManager.Generic import GenericArgument
from plasTeX.ConfigManager.String import StringOption
from plasTeX.ConfigManager import GetoptError
import collections


class UnknownCompoundGroup(GetoptError):
    """ Exception for an unknown grouping character used for a compound """
    def __init__(self, msg=''):
        GetoptError.__init__(self, msg, '')


class CompoundParser:
   """
   Compound configuration option

   Compound options are options grouped by a pair of grouping characters
   (e.g. '()', '[]', '{}', '<>').  The content between the grouping
   characters can be anything including other command line arguments.
   All content between the grouping characters will be unparsed.

   """

   def getArgument(self, args):
      """ Parse a compound argument """

      groups = {'<':'>', '(':')', '[':']', '{':'}'}

      # Determine grouping characters
      begin = args[0].strip()[0]
      try:
          end = groups[args[0].strip()[0]]
      except KeyError as info:
          name = self.name
          if self.actual: name = self.actual
          raise UnknownCompoundGroup(
                "Unknown compound grouping character '%s' in option '%s'" % (info, name))

      new_args = []
      while args and args[0].strip()[-1] != end:
          new_args.append(args.pop(0).strip())
      new_args.append(args.pop(0).strip())

      # Strip delimiters
      new_args[0] = new_args[0][1:]
      new_args[-1] = new_args[-1][:-1]

      output = []
      for item in new_args:
          item = item.strip()
          if not item:
              continue
          if ' ' in item:
              item = "'%s'" % item
          output.append(item)

      value = '%s %s %s' % (begin, ' '.join(output), end)

      return value, args


class CompoundOption(CompoundParser, StringOption):
   """
   Compound configuration option

   Compound options are options grouped by a pair of grouping characters
   (e.g. '()', '[]', '{}', '<>').  The content between the grouping
   characters can be anything including other command line arguments.
   All content between the grouping characters will be unparsed.

   """
   REGEX = re.compile(r'^(\s*(?:\(|\[|\{|\<)\s*)(.*)(\s*(?:\)|\]|\}|\]>)\s*)$')
   synopsis = "[ ... ]"

   def cast(self, data):
      if data is None: return
      return '%s %s %s' % (self.REGEX.sub(r'\1', str(data).strip()).strip(),
                           self.REGEX.sub(r'\2', str(data).strip()),
                           self.REGEX.sub(r'\3', str(data).strip()).strip())

   def __len__(self):
      if self.data is None:
          return 0
      else:
          return len(self.data)

   def __iadd__(self, other):
      if isinstance(self.callback, collections.Callable):
         other = self.callback(self.cast(other))

      if self.data is None:
         self.data = self.cast(other)
      else:
         other = self.REGEX.sub(r'\2', self.cast(other)).strip()
         self.data = self.REGEX.sub(r'\1\2 %s \3' % other, self.data)

      return self

class CompoundArgument(GenericArgument, CompoundOption):
   """ Compound command-line argument """



#!/usr/bin/env python

"""
C.5 Classes, Packages, and Page Styles (p176)

"""
import sys, os
from plasTeX import Command, Environment, DimenCommand, Token
from plasTeX.Logging import getLogger

# Put the plasTeX packages into the path
from plasTeX import Packages
sys.path.append(os.path.dirname(Packages.__file__))
del Packages

log = getLogger()
status = getLogger('status')

class PackageLoader(Command):
    extension = '.sty'
    def load(self, tex, file, options=None):
        try:
            self.ownerDocument.context.loadPackage(
                    tex, file+self.extension, options or {})
        except Exception as msg:
            log.error('Could not load package "%s": %s' % (file, msg))

#
# C.5.1 Document Class
#

class documentclass(PackageLoader):
    args = '[ options:dict ] name:str'
    extension = '.cls'
    def invoke(self, tex):
        a = self.parse(tex)
        self.load(tex, a['name'], a['options'])
        packages = self.ownerDocument.context.packages
        if a['name'] in list(packages.keys()):
            packages['documentclass'] = packages[a['name']]

class documentstyle(documentclass):
    pass

#
# Style Parameters
#

class bibindent(DimenCommand):
    value = DimenCommand.new(0)

class columnsep(DimenCommand):
    value = DimenCommand.new(0)

class columnseprule(DimenCommand):
    value = DimenCommand.new(0)

class mathindent(DimenCommand):
    value = DimenCommand.new(0)

#
# C.5.2 Packages
#

class usepackage(PackageLoader):
    args = '[ options:dict ] names:list:str'
    extension = '.sty'
    def invoke(self, tex):
        # Allow & in option names (this happens in natbib)
        catcode = self.ownerDocument.context.whichCode('&')
        self.ownerDocument.context.catcode('&', Token.CC_LETTER)
        a = self.parse(tex)
        self.ownerDocument.context.catcode('&', catcode)
        for fname in a['names']:
            self.load(tex, fname, a['options'])

class RequirePackage(usepackage):
    pass

#
# C.5.3 Page Styles
#

class pagestyle(Command):
    args = 'style:str'

class thispagestyle(pagestyle):
    pass

class markright(Command):
    args = 'text'

class markboth(Command):
    args = 'left right'

class pagenumbering(Command):
    args = 'style:str'

class twocolumn(Command):
    args = '[ text ]'

class onecolumn(Command):
    pass

#
# Style Parameters
#

# Figure C.3: Page style parameters

class paperheight(DimenCommand):
    value = DimenCommand.new('11in')

class paperwidth(DimenCommand):
    value = DimenCommand.new('8.5in')

class oddsidemargin(DimenCommand):
    value = DimenCommand.new('1in')

class evensidemargin(DimenCommand):
    value = DimenCommand.new('1in')

class textheight(DimenCommand):
    value = DimenCommand.new('9in')

class textwidth(DimenCommand):
    value = DimenCommand.new('6.5in')

class topmargin(DimenCommand):
    value = DimenCommand.new(0)

class headheight(DimenCommand):
    value = DimenCommand.new('0.5in')

class headsep(DimenCommand):
    value = DimenCommand.new('0.25in')

class footskip(DimenCommand):
    value = DimenCommand.new('0.5in')

class marginparsep(DimenCommand):
    value = DimenCommand.new('0.25in')

class marginparwidth(DimenCommand):
    value = DimenCommand.new('0.75in')

class topskip(DimenCommand):
    value = DimenCommand.new(0)

#
# C.5.4 The Title Page and Abstract
#

class maketitle(Command):
    blockType = True

class title(Command):
    args = '[ toc ] self'
    def invoke(self, tex):
        Command.invoke(self, tex)
        if 'title' not in list(self.ownerDocument.userdata.keys()):
            self.ownerDocument.userdata['title'] = self

class author(Command):
    args = 'self'
    def invoke(self, tex):
        Command.invoke(self, tex)
        userdata = self.ownerDocument.userdata
        if userdata.get('author') is None:
            userdata['author'] = []
        userdata['author'].append(self)

class date(Command):
    args = 'self'
    def invoke(self, tex):
        Command.invoke(self, tex)
        self.ownerDocument.userdata['date'] = self

class thanks(Command):
    args = 'self'
    def invoke(self, tex):
        Command.invoke(self, tex)
        self.ownerDocument.userdata['thanks'] = self

class abstract(Environment):
    blockType = True

class titlepage(Environment):
    blockType = True


#
# Extras...
#
class ProvidesPackage(Command):
    args = 'name [ message ]'

class ProvidesClass(Command):
    pass

class DeclareOption(Command):
    args = 'name:str [ default:nox ] value:nox'

class PackageWarning(Command):
    args = 'name:str message:str'

class ProcessOptions(Command):
    args = '*'

class LoadClass(usepackage):
    args = '[ options:dict ] names:list:str'
    extension = '.cls'

class NeedsTeXFormat(Command):
    args = 'name:str date:str'

class InputIfFileExists(Command):
    args = 'file:str true:nox false:nox'
    def invoke(self, tex):
        a = self.parse(tex)
        try:
            tex.input(tex.kpsewhich(a['file']))
            tex.pushTokens(a['true'])
        except (IOError, OSError):
            tex.pushTokens(a['false'])
        return []

class IfFileExists(Command):
    args = 'file:str true:nox false:nox'
    def invoke(self, tex):
        a = self.parse(tex)
        try:
            tex.kpsewhich(a['file'])
            tex.pushTokens(a['true'])
        except (IOError, OSError):
            tex.pushTokens(a['false'])
        return []



#!/usr/bin/env python

from plasTeX import Environment
from plasTeX.Base.LaTeX.Floats import Caption

class wrapfigure(Environment):
    args = '[ lines ] place:str [ overhang ] width'
    float = 'left'
    def invoke(self, tex):
        res = Environment.invoke(self, tex)
        if self.macroMode == self.MODE_BEGIN:
            a = self.attributes['place'].lower()
            if a in ['r','o']:
                self.float = 'right'
        return res
    
    class caption(Caption):
        counter = 'figure'

class wraptable(wrapfigure):
    pass


#!/usr/bin/env python

from plasTeX.ConfigManager.Generic import *
from plasTeX.ConfigManager import InvalidOptionError

class BooleanParser(GenericParser): pass

class BooleanOption(BooleanParser, GenericOption):
   """ Boolean configuration option """

   def cast(self, data):
      if data is None: return

      values = {'on':1,'off':0,'true':1,'false':0,'yes':1,'no':0}

      try:
         bool = int(data)
         if bool in [0, 1]:
             return bool
      except:
         try:
            bool = values[str(data).lower()]
            if bool in [0, 1]:
                return bool
         except: pass
      name = self.name
      if self.actual: name = self.actual
      raise InvalidOptionError(name, data, type='boolean')

   def validate(self, data):
      return self.cast(data)

   def defaultValue(self):
      if self.default:
          return 'yes'
      return 'no'

   def __repr__(self):
      """ Return command-line syntax as entered """
      if self.actual: return self.actual
      options = self.getPossibleOptions()
      negative = [x.replace('!','',1) for x in options if x.startswith('!')]
      positive = [x for x in options if not x.startswith('!')]
      if self.data and positive:
         return positive[0]
      elif not(self.data) and negative:
         return negative[0]
      return ''

   def __str__(self):
      """ Return string representation of the current option value """
      value = self.getValue()
      if value is not None:
         if value: return 'on'
         else: return 'off'
      return ''

   def acceptsArgument(self):
      """ Return a boolean indicating if the option accepts an argument """
      return 0

   def requiresArgument(self):
      """ Return a boolean indicating if the option requires an argument """
      return 0


class BooleanArgument(GenericArgument, BooleanOption):
   """ Boolean command-line option """



#!/usr/bin/env python

from plasTeX import Command
from plasTeX.Tokenizer import Token, Other

class MakeShortVerb(Command):
    args = 'char:cs'
    def invoke(self, tex):
        # Parse arguments
        res = Command.invoke(self, tex)
        # Get the specified character from the command sequence in
        # the `char` attribute
        char = str(self.attributes['char'].macroName)
        # Set the specified character as active
        self.ownerDocument.context.catcode(char, Token.CC_ACTIVE)
        # Create a new macro for the active character that calls _ShortVerb
        newclass = type('active::%s' % char, (_ShortVerb,),{})
        # Add the new macro to the global namespace
        self.ownerDocument.context['active::%s' % char] = newclass
        return res

class _ShortVerb(Command):
    """ Command to handle short verbatims """
    def invoke(self, tex):
        # Push the active character back to the input stream as
        # an Other token
        tex.pushToken(Other(self.nodeName.split('::')[-1]))
        # Instantiate a `verb` macro and let it take over the parsing
        # from here.  Also, return its return value.
        return self.ownerDocument.createElement('verb').invoke(tex)



#!/usr/bin/env python
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

templates = ['*.html','*.htm','*.xml','*.zpt','*.zpts','*.jinja2','*.jinja2s']
images = ['*.gif','*.png','*.jpg','*.jpeg','*.js','*.htc','*.svg']
styles = ['*.css']
javascript = ['*.js']

setup(name="plasTeX-gerby",
      description="LaTeX document processing framework",
      version="1.0",
      author="Kevin D. Smith",
      author_email="Kevin.Smith@sas.com",
      #url="",
      packages = [
         'plasTeX',
         'plasTeX.Base',
         'plasTeX.Base.LaTeX',
         'plasTeX.Base.TeX',
         'plasTeX.ConfigManager',
         'plasTeX.DOM',
         'plasTeX.Imagers',
         'plasTeX.Packages',
         'plasTeX.Renderers',

         'plasTeX.Renderers.Gerby',

         'plasTeX.Renderers.XHTML',
         'plasTeX.Renderers.XHTML.Themes',
         'plasTeX.Renderers.XHTML.Themes.default',
         'plasTeX.Renderers.XHTML.Themes.default.icons',
         'plasTeX.Renderers.XHTML.Themes.default.styles',
         'plasTeX.Renderers.XHTML.Themes.python',
         'plasTeX.Renderers.XHTML.Themes.python.icons',
         'plasTeX.Renderers.XHTML.Themes.plain',
         'plasTeX.Renderers.XHTML.Themes.minimal',
         'plasTeX.Renderers.HTML5',
         'plasTeX.Renderers.HTML5.Themes',
         'plasTeX.Renderers.HTML5.Themes.default',
         'plasTeX.Renderers.HTML5.Themes.default.styles',
         'plasTeX.Renderers.HTML5.Themes.default.js',
         'plasTeX.Renderers.DocBook',
         'plasTeX.Renderers.DocBook.Themes.default',
         'plasTeX.Renderers.DocBook.Themes.book',
         'plasTeX.Renderers.DocBook.Themes.article',
         'plasTeX.Renderers.ManPage',
         'plasTeX.Renderers.Text',
         'plasTeX.Renderers.ZPT',
         'plasTeX.Renderers.PageTemplate',
         'plasTeX.Renderers.PageTemplate.simpletal',
         'plasTeX.Renderers.S5',
         'plasTeX.Renderers.S5.Themes',
         'plasTeX.Renderers.S5.Themes.default',
         'plasTeX.Renderers.S5.Themes.default.ui',
         'plasTeX.Renderers.S5.Themes.default.ui.default',
      ],
      package_data = {
         'plasTeX': ['*.xml', 'plasTeXrc'],
         'plasTeX.Base.LaTeX': ['*.xml','*.txt'],
         'plasTeX.Renderers.DocBook': templates,
         'plasTeX.Renderers.DocBook.Themes.default': templates,
         'plasTeX.Renderers.DocBook.Themes.book': templates,
         'plasTeX.Renderers.DocBook.Themes.article': templates,

         'plasTeX.Renderers.Gerby': templates,

         'plasTeX.Renderers.XHTML': templates,
         'plasTeX.Renderers.XHTML.Themes.default': templates,
         'plasTeX.Renderers.XHTML.Themes.default.icons': images,
         'plasTeX.Renderers.XHTML.Themes.default.styles': styles,
         'plasTeX.Renderers.XHTML.Themes.python': templates+styles,
         'plasTeX.Renderers.XHTML.Themes.python.icons': images,
         'plasTeX.Renderers.XHTML.Themes.plain': templates,
         'plasTeX.Renderers.HTML5': templates,
         'plasTeX.Renderers.HTML5.Themes.default': templates+images,
         'plasTeX.Renderers.HTML5.Themes.default.styles': styles,
         'plasTeX.Renderers.HTML5.Themes.default.js': javascript,
         'plasTeX.Renderers.S5': templates,
         'plasTeX.Renderers.S5.Themes.default': templates,
         'plasTeX.Renderers.S5.Themes.default.ui.default': templates+styles+images,
      },
      scripts=['plasTeX/plastex'],
)



#!/usr/bin/env python

from plasTeX.Imagers import Imager as _Imager

class DVIPNG(_Imager):
    """ Imager that uses dvipng """
    command = 'dvipng -o img%d.png -D 120 -Q 4'
    fileExtension = '.png'

    def formatConfigOptions(self, config):
        options = []
        if config['resolution']:
            options.append(('-D', config['resolution']))
        return options

Imager = DVIPNG



#!/usr/bin/env python

"""
C.2 The Structure of the Document (p170)

"""

from plasTeX import Command, Environment
from plasTeX.Base.LaTeX.Sectioning import SectionUtils

class document(Environment, SectionUtils):
    level = Environment.DOCUMENT_LEVEL

    @property
    def title(self):
        return self.ownerDocument.userdata.get('title','')

    def invoke(self, tex):
        res = Environment.invoke(self, tex)

        # Set initial counter values
        if 'counters' in self.config:
            counters = self.config['counters']
            for name in list(counters.keys()):
                if name.startswith(';'):
                    continue
                try:
                    self.ownerDocument.context.counters[name].setcounter(counters[name]-1)
                except TypeError:
                    self.ownerDocument.context.counters[name].setcounter(int(counters[name])-1)

        return res
 
    @property
    def index(self):
        idx = self.getElementsByTagName(['theindex','printindex'])
        if idx:
            return idx[0]
        return []

class AtEndDocument(Command):
    args = 'commands:nox'

class AtBeginDocument(Command):
    args = 'commands:nox'



#!/usr/bin/env python

"""
C.9 Figures and Other Floating Bodies (p196)

"""

from plasTeX import Command, Environment, subclasses
from plasTeX import GlueCommand, DimenCommand
from plasTeX.Logging import getLogger
from plasTeX.Base.LaTeX.Arrays import Array


class Caption(Command):
    args = '[ toc ] self'
    blockType = True
    # Is the caption attached to an object?
    attached = False

    def invoke(self, tex):
        res = Command.invoke(self, tex)
        self.title = self.captionName
        return res

#
# C.9.1 Figures and Tables
#

class Float(Environment):
    blockType = True
    forcePars = True
    args = '[ loc:str ]'

    def digest(self, tokens):
        res = Environment.digest(self, tokens)
        # Apply captions to objects
        if self.macroMode == self.MODE_BEGIN:
            # Locate all caption nodes and nodes that are
            # capable of being captioned.
            all = self.allChildNodes
            captions = [x for x in all if isinstance(x, (Caption, Array.caption))]
            objects = [x for x in all if getattr(x, 'captionable', False)]
            # If there is only one caption, apply it to the float
            if len(captions) == 1:
                captions[0].attached = True
                self.title = captions[0]
            # If there are the same number of captions as there are
            # captionable items, apply the captions to the objects.
            if len(captions) == len(objects):
                while captions and objects:
                    captions[0].attached = True
                    objects.pop(0).title = captions.pop(0)
        return res

class figure(Float):
    class caption(Caption):
        counter = 'figure'
        templateName = 'nestedfigurecaption'

class FigureStar(figure):
    macroName = 'figure*'

class table(Float):
    captionable = []
    forcePars = False

    class caption(Caption):
        counter = 'table'
        templateName = 'nestedtablecaption'

    class tabular(Array):
        args = '[ pos:str ] colspec:nox'
        templateName = 'nestedtabular'

    class TabularStar(tabular):
        macroName = 'tabular*'
        args = 'width:dimen [ pos:str ] colspec:nox'
        templateName = 'nestedtabular'

    class tabularx(Array):
        args = 'width:nox colspec:nox'
        templateName = 'nestedtabular'

    class tabulary(Array):
        args = 'width:nox colspec:nox'
        templateName = 'nestedtabular'

class TableStar(table):
    macroName = 'table*'

class suppressfloats(Command):
    pass

# Counters

class topfraction(Command):
    str = '0.25'

class bottomfraction(Command):
    str = '0.25'

class textfraction(Command):
    str = '0.25'

class floatpagefraction(Command):
    str = '0.25'

class dbltopfraction(Command):
    str = '0.25'

class dblfloatpagefraction(Command):
    str = '0.25'

class floatsep(GlueCommand):
    value = GlueCommand.new(0)

class textfloatsep(GlueCommand):
    value = GlueCommand.new(0)

class intextsep(GlueCommand):
    value = GlueCommand.new(0)

class dblfloatsep(GlueCommand):
    value = GlueCommand.new(0)

class dbltextfloatsep(GlueCommand):
    value = GlueCommand.new(0)


#
# C.9.2 Marginal Notes
#

class marginpar(Command):
    args = '[ left ] right'

class reversemarginpar(Command):
    pass

class normalmarginpar(Command):
    pass

# Style Parameters

class marginparwidth(DimenCommand):
    value = DimenCommand.new(0)

class marginparsep(DimenCommand):
    value = DimenCommand.new(0)

class marginparpush(DimenCommand):
    value = DimenCommand.new(0)
